/*
  defaults.h - defaults settings configuration file
  Part of Grbl

  Copyright (c) 2017-2018 Gauthier Briere
  Copyright (c) 2012-2016 Sungeun K. Jeon for Gnea Research LLC

  Grbl is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Grbl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Grbl.  If not, see <http://www.gnu.org/licenses/>.
*/

/* The defaults.h file serves as a central default settings selector for different machine
   types, from DIY CNC mills to CNC conversions of off-the-shelf machines. The settings
   files listed here are supplied by users, so your results may vary. However, this should
   give you a good starting point as you get to know your machine and tweak the settings for
   your nefarious needs.
   NOTE: Ensure one and only one of these DEFAULTS_XXX values is defined in config.h */

#ifndef defaults_h

#ifdef DEFAULTS_GENERIC
  // Grbl generic default settings. Should work across different machines.
  #define DEFAULT_AXIS1_STEPS_PER_UNIT 250.0
  #define DEFAULT_AXIS2_STEPS_PER_UNIT 250.0
  #define DEFAULT_AXIS3_STEPS_PER_UNIT 250.0
  #define DEFAULT_AXIS1_MAX_RATE 500.0 // mm/min
  #define DEFAULT_AXIS2_MAX_RATE 500.0 // mm/min
  #define DEFAULT_AXIS3_MAX_RATE 500.0 // mm/min
  #define DEFAULT_AXIS1_ACCELERATION (10.0*60*60) // 10*60*60 mm/min^2 = 10 mm/sec^2
  #define DEFAULT_AXIS2_ACCELERATION (10.0*60*60) // 10*60*60 mm/min^2 = 10 mm/sec^2
  #define DEFAULT_AXIS3_ACCELERATION (10.0*60*60) // 10*60*60 mm/min^2 = 10 mm/sec^2
  #define DEFAULT_AXIS1_MAX_TRAVEL 2000.0 // mm NOTE: Must be a positive value.
  #define DEFAULT_AXIS2_MAX_TRAVEL 2000.0 // mm NOTE: Must be a positive value.
  #define DEFAULT_AXIS3_MAX_TRAVEL 100.0 // mm NOTE: Must be a positive value.
  #define DEFAULT_SPINDLE_RPM_MAX 1000.0 // rpm
  #define DEFAULT_SPINDLE_RPM_MIN 0.0 // rpm
  #define DEFAULT_STEP_PULSE_MICROSECONDS 10
  #define DEFAULT_STEPPING_INVERT_MASK 0
  #define DEFAULT_DIRECTION_INVERT_MASK 0
  #define DEFAULT_STEPPER_IDLE_LOCK_TIME 25 // msec (0-254, 255 keeps steppers enabled)
  #define DEFAULT_STATUS_REPORT_MASK 1 // MPos enabled
  #define DEFAULT_JUNCTION_DEVIATION 0.01 // mm
  #define DEFAULT_ARC_TOLERANCE 0.002 // mm
  #define DEFAULT_REPORT_INCHES 0 // false
  #define DEFAULT_INVERT_ST_ENABLE 0 // false
  #define DEFAULT_INVERT_LIMIT_PINS 1 // false
  #define DEFAULT_SOFT_LIMIT_ENABLE 0 // false
  #define DEFAULT_HARD_LIMIT_ENABLE 0  // false
  #define DEFAULT_INVERT_PROBE_PIN 0 // false
  #define DEFAULT_LASER_MODE 1 // false
  #define DEFAULT_HOMING_ENABLE 0  // false
  #define DEFAULT_HOMING_DIR_MASK 0 // move positive dir
  #define DEFAULT_HOMING_FEED_RATE 5.0 // mm/min
  #define DEFAULT_HOMING_SEEK_RATE 10.0 // mm/min
  #define DEFAULT_HOMING_DEBOUNCE_DELAY 1000 // msec (0-65k)
  #define DEFAULT_HOMING_PULLOFF 10.0 // mm
#endif


//#ifdef DEFAULTS_RAMPS_BOARD
  #define PLASMA_THC 1
  #define MICROSTEPS_AXIS1 16     // Microstepping = 1/4 pas
  #define STEP_REVS_AXIS1 200    // Moteurs à 200 pas par tour
  #define UNIT_PER_REV_AXIS1 10.0 // 10mm
  #define DEFAULT_AXIS1_STEPS_PER_UNIT (MICROSTEPS_AXIS1*STEP_REVS_AXIS1/UNIT_PER_REV_AXIS1) // 400
  #define MICROSTEPS_AXIS2 16     // Microstepping = 1/4 pas
  #define STEP_REVS_AXIS2 200    // Moteurs à 200 pas par tour
  #define UNIT_PER_REV_AXIS2 10.0 // 2mm
  #define DEFAULT_AXIS2_STEPS_PER_UNIT (MICROSTEPS_AXIS2*STEP_REVS_AXIS2/UNIT_PER_REV_AXIS2)
  #define MICROSTEPS_AXIS3 16     // Microstepping = 1/4 pas
  #define STEP_REVS_AXIS3 200    // Moteurs à 200 pas par tour
  #define UNIT_PER_REV_AXIS3 10.0 // 2mm
  #define DEFAULT_AXIS3_STEPS_PER_UNIT (MICROSTEPS_AXIS3*STEP_REVS_AXIS3/UNIT_PER_REV_AXIS3)

// cloned X
  #define MICROSTEPS_AXIS4 16     // Microstepping = 1/4 pas
  #define STEP_REVS_AXIS4 200    // Moteurs à 200 pas par tour
  #define UNIT_PER_REV_AXIS4 10.0 // 2mm
//  #define DEFAULT_AXIS4_STEPS_PER_UNIT DEFAULT_AXIS1_STEPS_PER_UNIT
  #define DEFAULT_AXIS4_STEPS_PER_UNIT (MICROSTEPS_AXIS4*STEP_REVS_AXIS4/UNIT_PER_REV_AXIS4)

  #define DEFAULT_AXIS5_STEPS_PER_UNIT 8.888889 // Direct drive : (200 pas par tours * 1/16 microsteps)/360°

  #define STEP_MAX_FREQUENCY 5000
  #define SECONDS_PER_MINUTE 60
  //#define DEFAULT_AXIS1_MAX_RATE 9000.0 // 9000 mm/min = 9000/60 = 150 mm/sec
  #define DEFAULT_AXIS1_MAX_RATE (STEP_MAX_FREQUENCY/DEFAULT_AXIS1_STEPS_PER_UNIT*SECONDS_PER_MINUTE) // 3000
  //#define DEFAULT_AXIS2_MAX_RATE 9000.0 // 9000 mm/min = 9000/60 = 150 mm/sec
  #define DEFAULT_AXIS2_MAX_RATE (STEP_MAX_FREQUENCY/DEFAULT_AXIS2_STEPS_PER_UNIT*SECONDS_PER_MINUTE)
  //#define DEFAULT_AXIS3_MAX_RATE 300.0  //  300 mm/min =  300/60 =   5 mm/sec
  #define DEFAULT_AXIS3_MAX_RATE (STEP_MAX_FREQUENCY/DEFAULT_AXIS3_STEPS_PER_UNIT*SECONDS_PER_MINUTE)

  #define DEFAULT_AXIS4_MAX_RATE DEFAULT_AXIS1_MAX_RATE
  #define DEFAULT_AXIS5_MAX_RATE 1440 // °/mn

  #define DEFAULT_AXIS1_ACCELERATION (50.0*60*60) // 300*60*60 mm/min^2 = 300 mm/sec^2
  #define DEFAULT_AXIS2_ACCELERATION (50.0*60*60) // 300*60*60 mm/min^2 = 300 mm/sec^2
  #define DEFAULT_AXIS3_ACCELERATION (50.0*60*60) // 100*60*60 mm/min^2 = 100 mm/sec^2
  #define DEFAULT_AXIS4_ACCELERATION DEFAULT_AXIS1_ACCELERATION
  #define DEFAULT_AXIS5_ACCELERATION (50.0*60*60) // 100*60*60 mm/min^2 = 100 mm/sec^2

  #define DEFAULT_AXIS1_MAX_TRAVEL 2000.0 // mm
  #define DEFAULT_AXIS2_MAX_TRAVEL 1750.0 // mm
  #define DEFAULT_AXIS3_MAX_TRAVEL 150.0 // mm
  #define DEFAULT_AXIS4_MAX_TRAVEL DEFAULT_AXIS1_MAX_TRAVEL
  #define DEFAULT_AXIS5_MAX_TRAVEL 360.0 // °

  #define DEFAULT_SPINDLE_RPM_MAX 12000 // rpm
  #define DEFAULT_SPINDLE_RPM_MIN 550.0 // rpm
  #define DEFAULT_STEP_PULSE_MICROSECONDS 10
  #define DEFAULT_STEPPING_INVERT_MASK 0
  #define DEFAULT_DIRECTION_INVERT_MASK 0
  #define DEFAULT_STEPPER_IDLE_LOCK_TIME 254 // msec (0-254, 255 keeps steppers enabled)
  #define DEFAULT_STATUS_REPORT_MASK 1 // MPos enabled
  #define DEFAULT_JUNCTION_DEVIATION 0.02 // mm
  #define DEFAULT_ARC_TOLERANCE 0.002 // mm
  #define DEFAULT_REPORT_INCHES 0 // false
  #define DEFAULT_INVERT_ST_ENABLE 0 // false
  #define DEFAULT_INVERT_LIMIT_PINS 0 // false
  #define DEFAULT_SOFT_LIMIT_ENABLE 1 // true
  #define DEFAULT_HARD_LIMIT_ENABLE 0  // false
  #define DEFAULT_INVERT_PROBE_PIN 0 // false
//  #define DEFAULT_LASER_MODE 0 // false
  #define DEFAULT_LASER_MODE 1 // false
  #define DEFAULT_HOMING_ENABLE 0  // true
  #define DEFAULT_HOMING_DIR_MASK 0 // move positive dir
  #define DEFAULT_HOMING_FEED_RATE 25.0 // mm/min
  #define DEFAULT_HOMING_SEEK_RATE 50.0 // mm/min
  #define DEFAULT_HOMING_DEBOUNCE_DELAY 1000 // msec (0-65k)
  #define DEFAULT_HOMING_PULLOFF 5.0 // mm
//#endif

#endif
